@@ -23,7 +23,7 @@ if MYPY:
 @signalemitter
 class Controller:
     """Glue class that holds the scene, (active) view(s), (active) tool(s) and possible user inputs.
-    
+
     The different types of views / tools / inputs are defined by plugins.
     :sa View
     :sa Tool
@@ -361,14 +361,14 @@ class Controller:
 
     toolOperationStarted = Signal()
     """Emitted whenever a tool starts a longer operation.
-    
+
     :param tool: The tool that started the operation.
     :sa Tool::startOperation
     """
 
     toolOperationStopped = Signal()
     """Emitted whenever a tool stops a longer operation.
-    
+
     :param tool: The tool that stopped the operation.
     :sa Tool::stopOperation
     """
@@ -498,7 +498,7 @@ class Controller:
     # Rotate camera view according defined angle
     def setCameraRotation(self, coordinate: str = "x", angle: int = 0) -> None:
         camera = self._scene.getActiveCamera()
-        if not camera:
+        if not camera or not self._camera_tool:
             return
         camera.setZoomFactor(camera.getDefaultZoomFactor())
         self._camera_tool.setOrigin(Vector(0, 100, 0))  # type: ignore
@@ -517,7 +517,7 @@ class Controller:
                 camera.setPosition(Vector(0, 100, 700))
                 self._camera_tool.rotateCamera(angle, 0)  # type: ignore
             elif coordinate == "y":
-                if angle == 90:
+                if angle % 360 == 90:
                     # Prepare the camera for top view, so no rotation has to be applied after setting the top view.
                     camera.setPosition(Vector(0, 100, 100))
                     self._camera_tool.rotateCamera(90, 0)  # type: ignore
@@ -526,6 +526,13 @@ class Controller:
                     self.setCameraOrigin("z")
                     camera.lookAt(Vector(0, 100, 1))
                     self._camera_tool.rotateCamera(0, 0)  # type: ignore
+                elif angle % 360 == 270:
+                    camera.setPosition(Vector(0, -100, 100))
+                    self._camera_tool.rotateCamera(90, 0)  # type: ignore
+                    camera.setPosition(Vector(0, -800, 1))
+                    self.setCameraOrigin("z")
+                    camera.lookAt(Vector(0, 100, 1))
+                    self._camera_tool.rotateCamera(0, 0)  # type: ignore
                 else:
                     camera.setPosition(Vector(0, 100, 700))
                     self._camera_tool.rotateCamera(0, angle)  # type: ignore
