@@ -1,11 +1,11 @@
-# Copyright (c) 2018 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 from typing import Any, Dict, List, Optional, Union, TYPE_CHECKING, cast
 
 from .FileReader import FileReader
 from .FileWriter import FileWriter
-from PyQt5.QtCore import QObject, pyqtProperty, pyqtSlot, QUrl
+from PyQt6.QtCore import QFileInfo, QObject, pyqtProperty, pyqtSlot, QUrl
 
 from UM.Logger import Logger
 from UM.Platform import Platform
@@ -17,16 +17,29 @@ i18n_catalog = i18nCatalog("uranium")
 if TYPE_CHECKING:
     from UM.Qt.QtApplication import QtApplication
 
-##  Central class for reading and writing meshes.
-#   This class is created by Application and handles reading and writing mesh files.
+
+def resolveAnySymlink(file_name: str) -> str:
+    """ Some OSs have different extensions for linked files (for example, one possibility is .lnk-files on Windows).
+    In such cases, the literal .lnk file will be attempted to open, not the file this resolves:
+    """
+
+    info = QFileInfo(file_name)
+    if info.exists() and info.isSymLink():  # Use 'exists': Not certain to be a literal file, depending on subclass.
+        file_name = info.symLinkTarget()
+    return file_name
+
+
 class FileHandler(QObject):
+    """Central class for reading and writing meshes.
+    This class is created by Application and handles reading and writing mesh files.
+    """
 
     def __init__(self, application: "QtApplication", writer_type: str = "unknown_file_writer", reader_type: str = "unknown_file_reader", parent: QObject = None) -> None:
         if cast(FileHandler, self.__class__).__instance is not None:
             raise RuntimeError("Try to create singleton '%s' more than once" % self.__class__.__name__)
-        cast(FileHandler, self.__class__).__instance = self
 
         super().__init__(parent)
+        cast(FileHandler, self.__class__).__instance = self
 
         self._application = application
         self._readers = {} # type: Dict[str, FileReader]
@@ -35,6 +48,8 @@ class FileHandler(QObject):
         self._writer_type = writer_type # type: str
         self._reader_type = reader_type # type: str
 
+        self._add_to_recent_files_hints = [] # type: List[QUrl]
+
         PluginRegistry.addType(self._writer_type, self.addWriter)
         PluginRegistry.addType(self._reader_type, self.addReader)
 
@@ -69,18 +84,23 @@ class FileHandler(QObject):
 
         return file_types
 
+    @pyqtSlot(QUrl, bool)
     @pyqtSlot(QUrl)
-    def readLocalFile(self, file: QUrl) -> None:
+    def readLocalFile(self, file: QUrl, add_to_recent_files_hint: bool = True) -> None:
         if not file.isValid():
             return
-        self._readLocalFile(file)
+        if add_to_recent_files_hint:
+            self._add_to_recent_files_hints.append(file)
+        self._readLocalFile(file, add_to_recent_files_hint)
 
-    def _readLocalFile(self, file: QUrl) -> None:
+    def _readLocalFile(self, file: QUrl, add_to_recent_files_hint: bool = True) -> None:
         raise NotImplementedError("_readLocalFile needs to be implemented by subclasses")
 
-    ##  Get list of all supported filetypes for writing.
-    #   \return List of dicts containing id, extension, description and mime_type for all supported file types.
     def getSupportedFileTypesWrite(self) -> List[Dict[str, Union[str, int]]]:
+        """Get list of all supported filetypes for writing.
+        :return: List of dicts containing id, extension, description and mime_type for all supported file types.
+        """
+
         supported_types = []
         meta_data = PluginRegistry.getInstance().getAllMetaData(filter={self._writer_type: {}}, active_only=True)
         for entry in meta_data:
@@ -125,15 +145,17 @@ class FileHandler(QObject):
     # \param file_name The name of the file to load.
     # \param kwargs Keyword arguments.
     # \returns None if nothing was found
-    def readerRead(self, reader: FileReader, file_name: str, **kwargs: Any):
+    def readerRead(self, reader, file_name: str, **kwargs: Any):
         raise NotImplementedError("readerRead must be implemented by subclasses.")
 
-    ##  Get a mesh writer object that supports writing the specified mime type
-    #
-    #   \param mime The mime type that should be supported.
-    #   \return A FileWriter instance or None if no mesh writer supports the specified mime type. If there are multiple
-    #           writers that support the specified mime type, the first entry is returned.
     def getWriterByMimeType(self, mime: str) -> Optional["FileWriter"]:
+        """Get a mesh writer object that supports writing the specified mime type
+
+        :param mime: The mime type that should be supported.
+        :return: A FileWriter instance or None if no mesh writer supports the specified mime type. If there are multiple
+        writers that support the specified mime type, the first entry is returned.
+        """
+
         writer_data = PluginRegistry.getInstance().getAllMetaData(filter={self._writer_type: {}}, active_only=True)
         for entry in writer_data:
             for output in entry[self._writer_type].get("output", []):
@@ -142,17 +164,21 @@ class FileHandler(QObject):
 
         return None
 
-    ##  Get an instance of a mesh writer by ID
     def getWriter(self, writer_id: str) -> Optional["FileWriter"]:
+        """Get an instance of a mesh writer by ID"""
+
         if writer_id not in self._writers:
             return None
 
         return self._writers[writer_id]
 
-    ##  Find a Reader that accepts the given file name.
-    #   \param file_name The name of file to load.
-    #   \returns Reader that accepts the given file name. If no acceptable Reader is found None is returned.
     def getReaderForFile(self, file_name: str) -> Optional["FileReader"]:
+        """Find a Reader that accepts the given file name.
+        :param file_name: The name of file to load.
+        :returns: Reader that accepts the given file name. If no acceptable Reader is found None is returned.
+        """
+
+        file_name = resolveAnySymlink(file_name)
         for id, reader in self._readers.items():
             try:
                 if reader.acceptsFile(file_name):
