@@ -1,4 +1,4 @@
-# Copyright (c) 2015 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 from UM.Message import Message
@@ -19,8 +19,8 @@ class ReadMeshJob(ReadFileJob):
     The result of this Job is a MeshData object.
     """
 
-    def __init__(self, filename: str) -> None:
-        super().__init__(filename)
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
         from UM.Qt.QtApplication import QtApplication
         self._application = QtApplication.getInstance()
         self._handler = QtApplication.getInstance().getMeshFileHandler()
@@ -29,6 +29,7 @@ class ReadMeshJob(ReadFileJob):
         super().run()
 
         if not self._result:
+            self._add_to_recent_files = False  # Failed to read any models due to error.
             self._result = []
 
         # Scale down to maximum bounds size if that is available
@@ -67,7 +68,10 @@ class ReadMeshJob(ReadFileJob):
                         scale_vector = Vector(scale_factor, scale_factor, scale_factor)
                         display_scale_factor = scale_factor * 100
 
-                        scale_message = Message(i18n_catalog.i18nc("@info:status", "Auto scaled object to {0}% of original size", ("%i" % display_scale_factor)), title = i18n_catalog.i18nc("@info:title", "Scaling Object"))
+                        scale_message = Message(i18n_catalog.i18nc("@info:status",
+                                                                   "Auto scaled model to {0}% of original size",
+                                                                   ("%i" % display_scale_factor)),
+                                                title = i18n_catalog.i18nc("@info:title", "Scaling Object"))
 
                         try:
                             node.scale(scale_vector)
