@@ -1,4 +1,4 @@
-# Copyright (c) 2021 Ultimaker B.V.
+# Copyright (c) 2022 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import imp
@@ -9,10 +9,10 @@ import stat  # For setting file permissions correctly;
 import time
 import types
 import zipfile
-from typing import Any, Callable, Dict, List, Optional, Tuple, TYPE_CHECKING
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple, TYPE_CHECKING
 
-from PyQt5.QtCore import QCoreApplication
-from PyQt5.QtCore import QObject, pyqtSlot, QUrl, pyqtProperty, pyqtSignal
+from PyQt6.QtCore import QCoreApplication
+from PyQt6.QtCore import QObject, pyqtSlot, QUrl, pyqtProperty, pyqtSignal
 
 from UM.CentralFileStorage import CentralFileStorage
 from UM.Logger import Logger
@@ -49,44 +49,46 @@ class PluginRegistry(QObject):
     def __init__(self, application: "Application", parent: QObject = None) -> None:
         if PluginRegistry.__instance is not None:
             raise RuntimeError("Try to create singleton '%s' more than once" % self.__class__.__name__)
-        PluginRegistry.__instance = self
 
         super().__init__(parent)
-        self.preloaded_plugins = []  # type: List[str]  # List of plug-in names that must be loaded before the rest, if the plug-ins are available. They are loaded in this order too.
+        PluginRegistry.__instance = self
 
-        self._application = application  # type: Application
-        self._api_version = application.getAPIVersion()  # type: Version
+        self.preloaded_plugins: List[str] = []  # List of plug-in names that must be loaded before the rest, if the plug-ins are available. They are loaded in this order too.
 
-        self._all_plugins = []        # type: List[str]
-        self._metadata = {}           # type: Dict[str, Dict[str, Any]]
+        self._application: Application = application
+        self._api_version: Version = application.getAPIVersion()
 
-        self._plugins_installed = []  # type: List[str]
+        self._all_plugins: List[str] = []
+        self._metadata: Dict[str, Dict[str, Any]] = {}
+
+        self._plugins_installed: List[str] = []
 
         # NOTE: The disabled_plugins and plugins_to_remove is explicitly set to None.
         # When actually loading the preferences, it's set to a list. This way we can see the
         # difference between no list and an empty one.
-        self._disabled_plugins = []  # type: List[str]
-        self._outdated_plugins = []  # type: List[str]
-        self._plugins_to_install = dict()  # type: Dict[str, Dict[str, str]]
-        self._plugins_to_remove = []  # type: List[str]
+        self._disabled_plugins: List[str] = []
+        self._outdated_plugins: List[str] = []
+        self._plugins_to_install: Dict[str, Dict[str, str]] = dict()
+        self._plugins_to_remove: List[str] = []
 
-        self._plugins = {}            # type: Dict[str, types.ModuleType]
-        self._found_plugins = {}      # type: Dict[str, types.ModuleType]  # Cache to speed up _findPlugin
-        self._plugin_objects = {}     # type: Dict[str, PluginObject]
+        self._plugins: Dict[str, types.ModuleType] = {}
+        self._found_plugins: Dict[str, types.ModuleType] = {}       # Cache to speed up _findPlugin
+        self._plugin_objects: Dict[str, PluginObject] = {}
 
-        self._plugin_locations = []  # type: List[str]
-        self._plugin_folder_cache = {}  # type: Dict[str, List[Tuple[str, str]]]  # Cache to speed up _locatePlugin
+        self._plugin_locations: List[str] = []
+        self._plugin_folder_cache: Dict[str, List[Tuple[str, str]]] = {}   # Cache to speed up _locatePlugin
 
-        self._bundled_plugin_cache = {}  # type: Dict[str, bool]
+        self._bundled_plugin_cache: Dict[str, bool] = {}
 
-        self._supported_file_types = {"umplugin": "Uranium Plugin"} # type: Dict[str, str]
+        self._supported_file_types: Dict[str, str] = {"umplugin": "Uranium Plugin"}
 
-        self._check_if_trusted = False  # type: bool
-        self._clean_hierarchy_sent_messages = []  # type: List[str]
-        self._debug_mode = False  # type: bool
-        self._checked_plugin_ids = []     # type: List[str]
-        self._distrusted_plugin_ids = []  # type: List[str]
-        self._trust_checker = None  # type: Optional[Trust]
+        self._check_if_trusted: bool = False
+        self._clean_hierarchy_sent_messages: List[str] = []
+        self._debug_mode: bool = False
+        self._checked_plugin_ids: List[str] = []
+        self._distrusted_plugin_ids: List[str] = []
+        self._trust_checker: Optional[Trust] = None
+        self._changed_activated_plugins_current_session: Set[str] = set()
 
     pluginRemoved = pyqtSignal(str)
 
@@ -107,7 +109,7 @@ class PluginRegistry(QObject):
 
         # File to store plugin info, such as which ones to install/remove and which ones are disabled.
         # At this point we can load this here because we already know the actual Application name, so the directory name
-        self._plugin_config_filename = os.path.join(os.path.abspath(config_path), "plugins.json") # type: str
+        self._plugin_config_filename: str = os.path.join(os.path.abspath(config_path), "plugins.json")
 
         from UM.Settings.ContainerRegistry import ContainerRegistry
         container_registry = ContainerRegistry.getInstance()
@@ -211,16 +213,28 @@ class PluginRegistry(QObject):
                 return False
         return True
 
+    pluginsEnabledOrDisabledChanged = pyqtSignal()
+
     #   Remove plugin from the list of enabled plugins and save to preferences:
     def disablePlugin(self, plugin_id: str) -> None:
         if plugin_id not in self._disabled_plugins:
             self._disabled_plugins.append(plugin_id)
+            if plugin_id not in self._changed_activated_plugins_current_session:
+                self._changed_activated_plugins_current_session.add(plugin_id)
+            else:
+                self._changed_activated_plugins_current_session.remove(plugin_id)
+            self.pluginsEnabledOrDisabledChanged.emit()
         self._savePluginData()
 
     #   Add plugin to the list of enabled plugins and save to preferences:
     def enablePlugin(self, plugin_id: str) -> None:
         if plugin_id in self._disabled_plugins:
             self._disabled_plugins.remove(plugin_id)
+            if plugin_id not in self._changed_activated_plugins_current_session:
+                self._changed_activated_plugins_current_session.add(plugin_id)
+            else:
+                self._changed_activated_plugins_current_session.remove(plugin_id)
+            self.pluginsEnabledOrDisabledChanged.emit()
         self._savePluginData()
 
     #   Get a list of enabled plugins:
@@ -253,6 +267,11 @@ class PluginRegistry(QObject):
     def getDisabledPlugins(self) -> List[str]:
         return self._disabled_plugins
 
+    def getCurrentSessionActivationChangedPlugins(self) -> Set[str]:
+        """Returns a set a plugins whom have changed their activation status in the current session, toggled between
+        en-/disabled after the last start-up status"""
+        return self._changed_activated_plugins_current_session
+
     #   Get a list of installed plugins:
     #   NOTE: These are plugins which have already been registered. This list is
     #         actually populated by the private _findInstalledPlugins() method.
@@ -493,7 +512,7 @@ class PluginRegistry(QObject):
         """Shows a message to the user remove the corrupted plugin"""
         message_text = i18n_catalog.i18nc("@error",
                                           "The plugin {} could not be loaded. Re-installing the plugin might solve "
-                                          "the issue", plugin_id)
+                                          "the issue.", plugin_id)
         unable_to_load_plugin_message = Message(text = message_text, message_type = Message.MessageType.ERROR, lifetime = 0)
         unable_to_load_plugin_message.addAction("remove",
                                name = i18n_catalog.i18nc("@action:button", "Remove plugin"),
@@ -510,6 +529,7 @@ class PluginRegistry(QObject):
     #   Uninstall a plugin with a given ID:
     @pyqtSlot(str, result = "QVariantMap")
     def uninstallPlugin(self, plugin_id: str) -> Dict[str, str]:
+        message_text = ""
         if plugin_id in self._plugins_to_install:
             del self._plugins_to_install[plugin_id]
             self._savePluginData()
@@ -561,8 +581,8 @@ class PluginRegistry(QObject):
                 for info in zip_ref.infolist():
                     extracted_path = zip_ref.extract(info.filename, path = plugin_folder)
                     permissions = os.stat(extracted_path).st_mode
-                    os.chmod(extracted_path, permissions | stat.S_IEXEC) # Make these files executable.
-        except: # Installing a new plugin should never crash the application.
+                    os.chmod(extracted_path, permissions | stat.S_IEXEC)  # Make these files executable.
+        except:  # Installing a new plugin should never crash the application.
             Logger.logException("e", "An exception occurred while installing plugin {path}".format(path = plugin_path))
 
         if plugin_id in self._disabled_plugins:
@@ -651,7 +671,7 @@ class PluginRegistry(QObject):
             highest_version = Version(0)
 
             for loc in locations:
-                meta_data = {}  # type: Dict[str, Any]
+                meta_data: Dict[str, Any]  = {}
                 plugin_location = os.path.join(loc, plugin_id)
                 metadata_file = os.path.join(plugin_location, "plugin.json")
                 try:
@@ -705,7 +725,7 @@ class PluginRegistry(QObject):
                 return None
 
         try:
-            module = imp.load_module(plugin_id, file, path, desc) #type: ignore #MyPy gets the wrong output type from imp.find_module for some reason.
+            module = imp.load_module(plugin_id, file, path, desc)  # type: ignore #MyPy gets the wrong output type from imp.find_module for some reason.
         except Exception:
             Logger.logException("e", "Import error loading module %s", plugin_id)
             return None
@@ -778,7 +798,7 @@ class PluginRegistry(QObject):
             raise InvalidMetaDataError(plugin_id)
         else:
             # Store the api_version as a Version object.
-            all_supported_sdk_versions = []  # type: List[Version]
+            all_supported_sdk_versions: List[Version] = []
             if "supported_sdk_versions" in meta_data["plugin"]:
                 all_supported_sdk_versions += [Version(supported_version) for supported_version in
                                                meta_data["plugin"]["supported_sdk_versions"]]
@@ -814,7 +834,7 @@ class PluginRegistry(QObject):
         location = os.path.join(location, plugin_id)
 
         try:
-            meta_data = plugin.getMetaData() #type: ignore #We catch the AttributeError that this would raise if the module has no getMetaData function.
+            meta_data = plugin.getMetaData()  # type: ignore #We catch the AttributeError that this would raise if the module has no getMetaData function.
             metadata_file = os.path.join(location, "plugin.json")
             try:
                 with open(metadata_file, "r", encoding = "utf-8") as file_stream:
@@ -922,8 +942,10 @@ class PluginRegistry(QObject):
 
         if not plugin:
             return None
-
-        path = os.path.dirname(self._plugins[plugin_id].__file__)
+        file_name = self._plugins[plugin_id].__file__
+        if file_name is None:
+            file_name = ""
+        path = os.path.dirname(file_name)
         if os.path.isdir(path):
             return path
 
@@ -959,8 +981,8 @@ class PluginRegistry(QObject):
         if plugin_type in cls._type_register_map:
             del cls._type_register_map[plugin_type]
 
-    _type_register_map = {}  # type: Dict[str, Callable[[Any], None]]
-    __instance = None    # type: PluginRegistry
+    _type_register_map: Dict[str, Callable[[Any], None]]  = {}
+    __instance = None  # type: PluginRegistry
 
     @classmethod
     def getInstance(cls, *args, **kwargs) -> "PluginRegistry":
