@@ -1,9 +1,9 @@
-# Copyright (c) 2015 Ultimaker B.V.
+# Copyright (c) 2022 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-from PyQt5.QtQml import qmlRegisterType, qmlRegisterSingletonType, qmlRegisterUncreatableType
+from PyQt6.QtQml import qmlRegisterType, qmlRegisterSingletonType, qmlRegisterUncreatableType
 
-from UM.Qt.Bindings import StageModel
+from UM.Qt.Bindings import StageModel, FileProviderModel, ProjectOutputDevicesModel
 from UM.Qt.Duration import Duration, DurationFormat
 
 from . import MainWindow
@@ -12,7 +12,6 @@ from . import ToolModel
 from . import ApplicationProxy
 from . import ControllerProxy
 from . import BackendProxy
-from . import SceneProxy
 from . import ResourcesProxy
 from . import OperationStackProxy
 from UM.Mesh.MeshFileHandler import MeshFileHandler
@@ -21,17 +20,16 @@ from . import PreferencesProxy
 from . import Theme
 from . import OpenGLContextProxy
 from . import PointingRectangle
+from UM.ColorImage import ColorImage
 from . import ActiveToolProxy
-from . import ActiveViewProxy
 from . import OutputDevicesModel
 from . import SelectionProxy
 from . import OutputDeviceManagerProxy
 from . import i18nCatalogProxy
 from . import ExtensionModel
-from . import PluginsModel
 from . import VisibleMessagesModel
-
-from . import MeshListModel
+from . import Utilities
+from . import TableModel
 
 from UM.Settings.Models.SettingDefinitionsModel import SettingDefinitionsModel
 from UM.Settings.Models.DefinitionContainersModel import DefinitionContainersModel
@@ -55,10 +53,6 @@ class Bindings:
     def createBackendProxy(self, engine, script_engine):
         return BackendProxy.BackendProxy()
 
-    @classmethod
-    def createSceneProxy(self, engine, script_engine):
-        return SceneProxy.SceneProxy()
-
     @classmethod
     def createResourcesProxy(cls, engine, script_engine):
         return ResourcesProxy.ResourcesProxy()
@@ -77,32 +71,30 @@ class Bindings:
         qmlRegisterType(ViewModel.ViewModel, "UM", 1, 0, "ViewModel")
         qmlRegisterType(ToolModel.ToolModel, "UM", 1, 0, "ToolModel")
         qmlRegisterType(PointingRectangle.PointingRectangle, "UM", 1, 0, "PointingRectangle")
+        qmlRegisterType(ColorImage, "UM", 1, 0, "ColorImage")
         qmlRegisterType(ExtensionModel.ExtensionModel, "UM", 1, 0, "ExtensionModel")
-        qmlRegisterType(PluginsModel.PluginsModel, "UM", 1, 0, "PluginsModel")
         qmlRegisterType(VisibleMessagesModel.VisibleMessagesModel, "UM", 1, 0, "VisibleMessagesModel")
 
         # Singleton proxy objects
-        qmlRegisterSingletonType(ControllerProxy.ControllerProxy, "UM", 1, 0, "Controller", Bindings.createControllerProxy)
-        qmlRegisterSingletonType(ApplicationProxy.ApplicationProxy, "UM", 1, 0, "Application", Bindings.createApplicationProxy)
-        qmlRegisterSingletonType(BackendProxy.BackendProxy, "UM", 1, 0, "Backend", Bindings.createBackendProxy)
-        qmlRegisterSingletonType(SceneProxy.SceneProxy, "UM", 1, 0, "Scene", Bindings.createSceneProxy)
-        qmlRegisterSingletonType(ResourcesProxy.ResourcesProxy, "UM", 1, 0, "Resources", Bindings.createResourcesProxy)
-        qmlRegisterSingletonType(OperationStackProxy.OperationStackProxy, "UM", 1, 0, "OperationStack", Bindings.createOperationStackProxy)
-        qmlRegisterSingletonType(MeshFileHandler, "UM", 1, 0, "MeshFileHandler", MeshFileHandler.getInstance)
-        qmlRegisterSingletonType(PreferencesProxy.PreferencesProxy, "UM", 1, 0, "Preferences", PreferencesProxy.createPreferencesProxy)
-        qmlRegisterSingletonType(Theme.Theme, "UM", 1, 0, "Theme", Theme.createTheme)
-        qmlRegisterSingletonType(ActiveToolProxy.ActiveToolProxy, "UM", 1, 0, "ActiveTool", ActiveToolProxy.createActiveToolProxy)
-        qmlRegisterSingletonType(ActiveViewProxy.ActiveViewProxy, "UM", 1, 0, "ActiveView", ActiveViewProxy.createActiveViewProxy)
-        qmlRegisterSingletonType(SelectionProxy.SelectionProxy, "UM", 1, 0, "Selection", SelectionProxy.createSelectionProxy)
+        qmlRegisterSingletonType(ControllerProxy.ControllerProxy, "UM", 1, 0, Bindings.createControllerProxy, "Controller")
+        qmlRegisterSingletonType(ApplicationProxy.ApplicationProxy, "UM", 1, 0, Bindings.createApplicationProxy, "Application")
+        qmlRegisterSingletonType(BackendProxy.BackendProxy, "UM", 1, 0, Bindings.createBackendProxy, "Backend")
+        qmlRegisterSingletonType(ResourcesProxy.ResourcesProxy, "UM", 1, 0, Bindings.createResourcesProxy, "Resources")
+        qmlRegisterSingletonType(OperationStackProxy.OperationStackProxy, "UM", 1, 0, Bindings.createOperationStackProxy, "OperationStack")
+        qmlRegisterSingletonType(MeshFileHandler, "UM", 1, 0, MeshFileHandler.getInstance, "MeshFileHandler")
+        qmlRegisterSingletonType(PreferencesProxy.PreferencesProxy, "UM", 1, 0, PreferencesProxy.createPreferencesProxy, "Preferences")
+        qmlRegisterSingletonType(Theme.Theme, "UM", 1, 0, Theme.createTheme, "Theme")
+        qmlRegisterSingletonType(ActiveToolProxy.ActiveToolProxy, "UM", 1, 0, ActiveToolProxy.createActiveToolProxy, "ActiveTool")
+        qmlRegisterSingletonType(SelectionProxy.SelectionProxy, "UM", 1, 0, SelectionProxy.createSelectionProxy, "Selection")
 
-        qmlRegisterUncreatableType(Duration, "UM", 1, 0, "Duration", "")
-        qmlRegisterUncreatableType(DurationFormat, "UM", 1, 0, "DurationFormat", "")
+        qmlRegisterUncreatableType(Duration, "UM", 1, 0, "", "Duration")
+        qmlRegisterUncreatableType(DurationFormat, "UM", 1, 0, "", "DurationFormat")
 
         # Additions after 15.06. Uses API version 1.1 so should be imported with "import UM 1.1"
         qmlRegisterType(OutputDevicesModel.OutputDevicesModel, "UM", 1, 1, "OutputDevicesModel")
         qmlRegisterType(i18nCatalogProxy.i18nCatalogProxy, "UM", 1, 1, "I18nCatalog")
 
-        qmlRegisterSingletonType(OutputDeviceManagerProxy.OutputDeviceManagerProxy, "UM", 1, 1, "OutputDeviceManager", OutputDeviceManagerProxy.createOutputDeviceManagerProxy)
+        qmlRegisterSingletonType(OutputDeviceManagerProxy.OutputDeviceManagerProxy, "UM", 1, 1, OutputDeviceManagerProxy.createOutputDeviceManagerProxy, "OutputDeviceManager")
 
         # Additions after 2.1. Uses API version 1.2
         qmlRegisterType(SettingDefinitionsModel, "UM", 1, 2, "SettingDefinitionsModel")
@@ -112,11 +104,24 @@ class Bindings:
         qmlRegisterType(SettingPropertyProvider, "UM", 1, 2, "SettingPropertyProvider")
         qmlRegisterType(SettingPreferenceVisibilityHandler, "UM", 1, 2, "SettingPreferenceVisibilityHandler")
         qmlRegisterType(ContainerPropertyProvider, "UM", 1, 2, "ContainerPropertyProvider")
-        qmlRegisterType(MeshListModel.MeshListModel, "UM", 1, 2, "MeshListModel")
 
         # Additions after 2.3;
-        qmlRegisterSingletonType(WorkspaceFileHandler, "UM", 1, 3, "WorkspaceFileHandler", WorkspaceFileHandler.getInstance)
-        qmlRegisterSingletonType(OpenGLContextProxy.OpenGLContextProxy, "UM", 1, 3, "OpenGLContextProxy", Bindings.createOpenGLContextProxy)
+        qmlRegisterSingletonType(WorkspaceFileHandler, "UM", 1, 3, WorkspaceFileHandler.getInstance, "WorkspaceFileHandler")
+        qmlRegisterSingletonType(OpenGLContextProxy.OpenGLContextProxy, "UM", 1, 3, Bindings.createOpenGLContextProxy, "OpenGLContextProxy")
 
         # Additions after 3.1
         qmlRegisterType(StageModel.StageModel, "UM", 1, 4, "StageModel")
+
+        # Additions after 4.6
+        qmlRegisterSingletonType(Utilities.UrlUtil, "UM", 1, 5, Utilities.createUrlUtil, "UrlUtil")
+
+        # Additions after 4.9
+        qmlRegisterType(FileProviderModel.FileProviderModel, "UM", 1, 6, "FileProviderModel")
+        qmlRegisterType(ProjectOutputDevicesModel.ProjectOutputDevicesModel, "UM", 1, 6, "ProjectOutputDevicesModel")
+
+        # Additions after 5.0
+        qmlRegisterType(TableModel.TableModel, "UM", 1, 6, "TableModel")
+
+    @staticmethod
+    def addRegisterType(class_type: type, qml_import_name: str, major_version: int, minor_version: int, class_name: str) -> None:
+        qmlRegisterType(class_type, qml_import_name, major_version, minor_version, class_name)
