@@ -1,14 +1,15 @@
-# Copyright (c) 2020 Ultimaker B.V.
+# Copyright (c) 2022 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import json
 import os
 import sys
+import warnings
 from typing import Dict, Optional, List
 
-from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal, QCoreApplication, QUrl, QSizeF
-from PyQt5.QtGui import QColor, QFont, QFontMetrics, QFontDatabase
-from PyQt5.QtQml import QQmlComponent, QQmlContext
+from PyQt6.QtCore import QObject, pyqtProperty, pyqtSignal, QCoreApplication, QUrl, QSizeF
+from PyQt6.QtGui import QColor, QFont, QFontMetrics, QFontDatabase
+from PyQt6.QtQml import QQmlComponent, QQmlContext
 
 import UM.Application
 from UM.FlameProfiler import pyqtSlot
@@ -21,9 +22,9 @@ class Theme(QObject):
         super().__init__(parent)
 
         self._engine = engine
-        self._styles = None  # type: Optional[QObject]
         self._path = ""
-        self._icons = {}  # type: Dict[str, QUrl]
+        self._icons = {}  # type: Dict[str, Dict[str, QUrl]]
+        self._deprecated_icons = {} # type: Dict[str, Dict[str, str]]
         self._images = {}  # type: Dict[str, QUrl]
 
         # Workaround for incorrect default font on Windows
@@ -49,7 +50,6 @@ class Theme(QObject):
     themeLoaded = pyqtSignal()
 
     def reload(self):
-        self._styles = None
         self._path = ""
         self._icons = {}
         self._images = {}
@@ -116,10 +116,32 @@ class Theme(QObject):
         Logger.log("w", "No size %s defined in Theme", size)
         return QSizeF()
 
+    @pyqtSlot(str, str, result = "QUrl")
     @pyqtSlot(str, result = "QUrl")
-    def getIcon(self, icon_name: str) -> QUrl:
-        if icon_name in self._icons:
-            return self._icons[icon_name]
+    def getIcon(self, icon_name: str, detail_level: str = "default") -> QUrl:
+        """
+        Finds and returns the url of the requested icon. The icons are organized in folders according to their detail
+        level and the same icon may exist with more details. If a detail level is not specified, the icon will be
+        retrieved from the "default" folder. Icons with a higher detail level are recommended to be used with a bigger
+        width/height.
+
+        :param icon_name: The name of the icon to be retrieved. The same icon may exist in multiple detail levels.
+        :param detail_level: The level of detail of the icon. Choice between "low, "default", "medium", "high".
+        :return: The file url of the requested icon, in the requested detail level.
+        """
+        if detail_level in self._icons:
+            if icon_name in self._icons[detail_level]:
+                return self._icons[detail_level][icon_name]
+        elif icon_name in self._icons["icons"]:  # Retrieve the "old" icon from the base icon folder
+            return self._icons["icons"][icon_name]
+
+        if icon_name in self._deprecated_icons:
+            new_icon = self._deprecated_icons[icon_name]["new_icon"]
+            warning = f"The icon '{icon_name}' is deprecated. Please use icon '{new_icon}' instead."
+
+            Logger.log("w_once", warning)
+            warnings.warn(warning, DeprecationWarning, stacklevel=2)
+            return self.getIcon(self._deprecated_icons[icon_name]["new_icon"], self._deprecated_icons[icon_name]["size"])
 
         # We don't log this anymore since we have new fallback behavior to load the icon from a plugin folder
         # Logger.log("w", "No icon %s defined in Theme", icon_name)
@@ -145,10 +167,6 @@ class Theme(QObject):
         Logger.log("w", "No font %s defined in Theme", font_name)
         return QFont()
 
-    @pyqtProperty(QObject, notify = themeLoaded)
-    def styles(self):
-        return self._styles
-
     @pyqtSlot(str)
     def load(self, path: str, is_first_call: bool = True) -> None:
         if path == self._path:
@@ -179,7 +197,23 @@ class Theme(QObject):
             pass  # No metadata or no inherits keyword in the theme.json file
 
         if "colors" in data:
-            for name, color in data["colors"].items():
+            for name, value in data["colors"].items():
+
+                if not is_first_call and isinstance(value, str):
+                    # Keep parent theme string colors as strings and parse later
+                    self._colors[name] = value
+                    continue
+
+                if isinstance(value, str) and is_first_call:
+                    # value is reference to base_colors color name
+                    try:
+                        color = data["base_colors"][value]
+                    except IndexError:
+                        Logger.log("w", "Colour {value} could not be found in base_colors".format(value = value))
+                        continue
+                else:
+                    color = value
+
                 try:
                     c = QColor(color[0], color[1], color[2], color[3])
                 except IndexError:  # Color doesn't have enough components.
@@ -187,6 +221,26 @@ class Theme(QObject):
                     continue  # Skip this one then.
                 self._colors[name] = c
 
+        if "base_colors" in data:
+            for name, color in data["base_colors"].items():
+                try:
+                    c = QColor(color[0], color[1], color[2], color[3])
+                except IndexError:  # Color doesn't have enough components.
+                    Logger.log("w", "Colour {name} doesn't have enough components. Need to have 4, but had {num_components}.".format(name = name, num_components = len(color)))
+                    continue  # Skip this one then.
+                self._colors[name] = c
+
+        if is_first_call and self._colors:
+            #Convert all string value colors to their referenced color
+            for name, color in self._colors.items():
+                if isinstance(color, str):
+                    try:
+                        c = self._colors[color]
+                        self._colors[name] = c
+                    except:
+                        Logger.log("w", "Colour {name} {color} does".format(name = name, color = color))
+
+
         fonts_dir = os.path.join(path, "fonts")
         if os.path.isdir(fonts_dir):
             for root, dirnames, filenames in os.walk(fonts_dir):
@@ -205,10 +259,10 @@ class Theme(QObject):
                 else:
                     q_font.setWeight(font.get("weight", 50))
 
-                q_font.setLetterSpacing(QFont.AbsoluteSpacing, font.get("letterSpacing", 0))
+                q_font.setLetterSpacing(QFont.SpacingType.AbsoluteSpacing, font.get("letterSpacing", 0))
                 q_font.setItalic(font.get("italic", False))
                 q_font.setPointSize(int(font.get("size", 1) * system_font_size))
-                q_font.setCapitalization(QFont.AllUppercase if font.get("capitalize", False) else QFont.MixedCase)
+                q_font.setCapitalization(QFont.Capitalization.AllUppercase if font.get("capitalize", False) else QFont.Capitalization.MixedCase)
 
                 self._fonts[name] = q_font
 
@@ -222,9 +276,26 @@ class Theme(QObject):
 
         iconsdir = os.path.join(path, "icons")
         if os.path.isdir(iconsdir):
-            for icon in os.listdir(iconsdir):
-                name = os.path.splitext(icon)[0]
-                self._icons[name] = QUrl.fromLocalFile(os.path.join(iconsdir, icon))
+            try:
+                for base_path, _, icons in os.walk(iconsdir):
+                    detail_level = base_path.split(os.sep)[-1]
+                    if detail_level not in self._icons:
+                        self._icons[detail_level] = {}
+                    for icon in icons:
+                        name = os.path.splitext(icon)[0]
+                        self._icons[detail_level][name] = QUrl.fromLocalFile(os.path.join(base_path, icon))
+            except EnvironmentError:  # Exception when calling os.walk, e.g. no access rights.
+                pass  # Won't get any icons then. Images will show as black squares.
+
+            deprecated_icons_file = os.path.join(iconsdir, "deprecated_icons.json")
+            if os.path.isfile(deprecated_icons_file):
+                try:
+                    with open(deprecated_icons_file, encoding="utf-8") as f:
+                        data = json.load(f)
+                        for icon in data:
+                            self._deprecated_icons[icon] = data[icon]
+                except (UnicodeDecodeError, json.decoder.JSONDecodeError, EnvironmentError):
+                    Logger.logException("w", "Could not parse deprecated icons list %s", deprecated_icons_file)
 
         imagesdir = os.path.join(path, "images")
         if os.path.isdir(imagesdir):
@@ -232,18 +303,8 @@ class Theme(QObject):
                 name = os.path.splitext(image)[0]
                 self._images[name] = QUrl.fromLocalFile(os.path.join(imagesdir, image))
 
-        styles = os.path.join(path, "styles.qml")
-        if os.path.isfile(styles):
-            c = QQmlComponent(self._engine, styles)
-            context = QQmlContext(self._engine, self._engine)
-            context.setContextProperty("Theme", self)
-            self._styles = c.create(context)
-
-            if c.isError():
-                for error in c.errors():
-                    Logger.log("e", error.toString())
-
         Logger.log("d", "Loaded theme %s", path)
+        Logger.info(f"System's em size is {self._em_height}px.")
         self._path = path
 
         # only emit the theme loaded signal once after all the themes in the inheritance chain have been loaded
@@ -253,7 +314,7 @@ class Theme(QObject):
     def _initializeDefaults(self) -> None:
         self._fonts = {
             "system": QCoreApplication.instance().font(),
-            "fixed": QFontDatabase.systemFont(QFontDatabase.FixedFont)
+            "fixed": QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
         }
 
         palette = QCoreApplication.instance().palette()
@@ -280,4 +341,3 @@ class Theme(QObject):
 
 def createTheme(engine, script_engine = None):
     return Theme.getInstance(engine)
-
