@@ -1,8 +1,9 @@
-# Copyright (c) 2019 Ultimaker B.V.
+# Copyright (c) 2022 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
-from typing import Union, Dict
 
-from PyQt5.QtCore import QObject, pyqtSignal, pyqtProperty, QUrl
+from typing import Union, Dict, Optional
+
+from PyQt6.QtCore import QObject, QUrl
 
 from UM.PluginObject import PluginObject
 
@@ -14,12 +15,10 @@ class Stage(QObject, PluginObject):
     Uranium has no notion of specific view locations as that's application specific.
     """
 
-    iconSourceChanged = pyqtSignal()
-
-    def __init__(self, parent = None) -> None:
+    def __init__(self, parent: Optional[QObject] = None) -> None:
         super().__init__(parent)
-        self._components = {}  # type: Dict[str, QUrl]
-        self._icon_source = QUrl()
+        self._components: Dict[str, QUrl] = {}
+        self._icon_source: QUrl = QUrl()
 
     def onStageSelected(self) -> None:
         """Something to do when this Stage is selected"""
@@ -42,15 +41,3 @@ class Stage(QObject, PluginObject):
         if name in self._components:
             return self._components[name]
         return QUrl()
-
-    @pyqtProperty(QUrl, notify = iconSourceChanged)
-    def iconSource(self) -> QUrl:
-        return self._icon_source
-
-    def setIconSource(self, source: Union[str, QUrl]) -> None:
-        if type(source) == str:
-            source = QUrl.fromLocalFile(source)
-
-        if self._icon_source != source:
-            self._icon_source = source
-            self.iconSourceChanged.emit()
