@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2022 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import sys
@@ -6,9 +6,9 @@ import os.path
 import signal
 import traceback
 
-from PyQt5.QtCore import QObject, QUrl, pyqtSlot, pyqtProperty, pyqtSignal
-from PyQt5.QtQml import QQmlApplicationEngine, qmlRegisterType
-from PyQt5.QtWidgets import QApplication
+from PyQt6.QtCore import QObject, QUrl, pyqtSlot, pyqtProperty, pyqtSignal
+from PyQt6.QtQml import QQmlApplicationEngine, qmlRegisterType
+from PyQt6.QtWidgets import QApplication
 
 import UM.Resources
 import UM.Settings
@@ -75,4 +75,4 @@ if file_name:
     engine.rootContext().setContextProperty("open_file", QUrl.fromLocalFile(file_name))
 
 engine.load(os.path.join(os.path.dirname(os.path.abspath(__file__)), "main.qml"))
-app.exec_()
+app.exec()
