@@ -1,12 +1,12 @@
-# Copyright (c) 2021 Ultimaker B.V.
+# Copyright (c) 2022 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import os
 import sys
 
-from PyQt5.QtCore import QUrl
-from PyQt5.QtGui import QDesktopServices
-from PyQt5.QtWidgets import QFileDialog, QMessageBox
+from PyQt6.QtCore import QUrl
+from PyQt6.QtGui import QDesktopServices
+from PyQt6.QtWidgets import QFileDialog, QMessageBox
 
 from UM.Application import Application
 from UM.FileHandler.WriteFileJob import WriteFileJob
@@ -56,14 +56,11 @@ class LocalFileOutputDevice(ProjectOutputDevice):
         dialog = QFileDialog()
 
         dialog.setWindowTitle(catalog.i18nc("@title:window", "Save to Disk"))
-        dialog.setFileMode(QFileDialog.AnyFile)
-        dialog.setAcceptMode(QFileDialog.AcceptSave)
+        dialog.setFileMode(QFileDialog.FileMode.AnyFile)
+        dialog.setAcceptMode(QFileDialog.AcceptMode.AcceptSave)
 
         # Ensure platform never ask for overwrite confirmation since we do this ourselves
-        dialog.setOption(QFileDialog.DontConfirmOverwrite)
-
-        if sys.platform == "linux" and "KDE_FULL_SESSION" in os.environ:
-            dialog.setOption(QFileDialog.DontUseNativeDialog)
+        dialog.setOption(QFileDialog.Option.DontConfirmOverwrite)
 
         filters = []
         mime_types = []
@@ -122,7 +119,7 @@ class LocalFileOutputDevice(ProjectOutputDevice):
         if selected_filter is not None:
             dialog.selectNameFilter(selected_filter)
 
-        if not dialog.exec_():
+        if not dialog.exec():
             raise OutputDeviceError.UserCanceledError()
 
         save_path = dialog.directory().absolutePath()
@@ -137,7 +134,7 @@ class LocalFileOutputDevice(ProjectOutputDevice):
 
         if os.path.exists(file_name):
             result = QMessageBox.question(None, catalog.i18nc("@title:window", "File Already Exists"), catalog.i18nc("@label Don't translate the XML tag <filename>!", "The file <filename>{0}</filename> already exists. Are you sure you want to overwrite it?").format(file_name))
-            if result == QMessageBox.No:
+            if result == QMessageBox.StandardButton.No:
                 raise OutputDeviceError.UserCanceledError()
 
         # Actually writing file
