@@ -1,10 +1,10 @@
-# Copyright (c) 2019 Ultimaker B.V.
+# Copyright (c) 2022 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import time
 from typing import cast, List, Optional, Union
 
-from PyQt5.QtCore import Qt, QTimer
+from PyQt6.QtCore import Qt, QTimer
 
 from UM.Event import Event, MouseEvent, KeyEvent
 from UM.Math.Float import Float
@@ -42,7 +42,7 @@ class TranslateTool(Tool):
         self._grid_size = 10
         self._moved = False
 
-        self._shortcut_key = Qt.Key_T
+        self._shortcut_key = Qt.Key.Key_T
 
         self._distance_update_time = None #type: Optional[float]
         self._distance = None #type: Optional[Vector]
